# Proximity Setup

## Purpose
Provide end-to-end instructions for setting up SubstratumNode for the purposes of Proximity Due Diligence.

## Server vs Client

For the purposes of this demo you will need at least three instances of SubstratumNode running through the server CLI to route traffic.  You then need one instance running a desktop client to be able to view the content you will route over the network.

## Client - building from source

1. Install Ubuntu 16.04 Server from source
2. Open Terminal
3. `mkdir substratum`
4. `cd substratum`
5. Install Git - `sudo apt install git`
6. Clone SubstratumNode source using your personal access token
    ```
    git clone https://{PERSONAL_ACCESS_TOKEN}@github.com/SubstratumNetwork/SubstratumNode.git
    ```
7. Install dependencies
    ```
    sudo apt install module-assistant dkms build-essential git libssl-dev pkg-config cmake curl gcc-arm-linux-gnueabihf; curl https://sh.rustup.rs -sSf | sh; grep -r "cargo test --release"; rustup target add arm-unknown-linux-gnueabihf
    ```
8. Configure your shell
    `source $HOME/.cargo/env`

9. Revert Rust to older version
    `rustup default 1.37.0`

10. Navigate into your SubstratumNode/dns_utility directory
    `cd SubstratumNode/dns_utility`

11. Build with Cargo
    ```
    cargo build --all --lib --bins --tests --release --verbose
    ```

12. Navigate into your SubstratumNode/node directory
    `cd ../node`

13. Build with Cargo
    ```
    cargo build --all --lib --bins --tests --release --verbose
    ```

14. Move executable files into single directory for ease of use
    ```
    mkdir $HOME/substratum/releases
    cp $HOME/substratum/SubstratumNode/dns_utility/target/release/dns_utility $HOME/substratum/releases/dns_utility
    cp $HOME/substratum/SubstratumNode/node/target/release/SubstratumNode $HOME/substratum/releases/SubstratumNode
    ```

15. Navigate to your new releases directory
    `cd $HOME/substratum/releases`

### Fix the Port 53 Bug

Documentation on this here https://github.com/SubstratumNetwork/SubstratumNode/blob/master/node/docs/PORT_53.md - but making it simple

1.  `sudo nano /etc/NetworkManager/NetworkManager.conf`
2.  Comment out `dns=dnsmasq`, making it `#dns=dnsmasq`
3.  Reboot your machine - `sudo reboot -i`

## Server - building from source
You need three server nodes to ensure enough nodes for CORES Protocol.

### Basic Setup

1. Install Ubuntu 16.04 Desktop from source
2. SSH into your server

### Notes for users connecting as root user

1. Change permissions on your SubstratumNode executable
    `chmod u+x SubstratumNode`

2. Add a non root user
    `sudo useradd node`

3. Give the new user sudo permissions
    `sudo usermod -aG sudo node`

4. Switch to the new node user
    `su node`

5. If not there already navigate to your home directory
    `cd $HOME`

### Build from Source

3. `mkdir substratum`
4. `cd substratum`
5. Install Git - `sudo apt install git`
6. Clone SubstratumNode source using your personal access token
    ```
    git clone https://{PERSONAL_ACCESS_TOKEN}@github.com/SubstratumNetwork/SubstratumNode.git
    ```
7. Install dependencies
    ```
    sudo apt install module-assistant dkms build-essential git libssl-dev pkg-config cmake curl gcc-arm-linux-gnueabihf && \
    curl https://sh.rustup.rs -sSf | sh && \
    source $HOME/.cargo/env && \
    grep -r "cargo test --release" && \
    rustup target add arm-unknown-linux-gnueabihf
    ```

9. Revert Rust to older version
    `rustup default 1.37.0`

10. Navigate into your SubstratumNode/node directory
    `cd ../node`

11. Build with Cargo
    ```
    cargo build --all --lib --bins --tests --release --verbose
    ```

12. Move executable files into single directory for ease of use
    ```
    mkdir $HOME/substratum/releases && \
    cp $HOME/substratum/SubstratumNode/dns_utility/target/release/dns_utility $HOME/substratum/releases/dns_utility && \
    cp $HOME/substratum/SubstratumNode/node/target/release/SubstratumNode $HOME/substratum/releases/SubstratumNode
    ```

13. Navigate to your new releases directory
    `cd $HOME/substratum/releases`

### Fix the Port 53 Bug

Documentation on this here https://github.com/SubstratumNetwork/SubstratumNode/blob/master/node/docs/PORT_53.md - but making it simple

1.  `sudo nano /etc/NetworkManager/NetworkManager.conf`
2.  Comment out `dns=dnsmasq`, making it `#dns=dnsmasq`
3.  Reboot your machine - `sudo reboot -i`

## Notes about blockchain integration for finance

Currently the Substratum network uses cryptocurrency as the underlying payment mechanism.  For the purposes of this demo we will be using the test Ethereum Network (Ropsten) to facilitate these transactions.  You can setup your own Ropsten nodes through Infura at https://infura.io but we have also provided Ropsten endpoints, earning wallets and consuming wallets (preloaded with HOT tokens) to make this easier.  If you wish to replace these with your own wallets there are instructions online on how to do this and this will be stripped out with the ARC integration.

## Cryptocurrency Details

### Node 1 - Bootstrap Node

* Earning Wallet: `0xB1E9823Cd2Fb8Bb33d506a672d8eE9E5b41f9472`
* Blockchain Service URL: https://ropsten.infura.io/v3/30913bd15f5f41bcb1620ec123df5380

### Node 2 - Serving Node

* Earning Wallet: `0xB1E9823Cd2Fb8Bb33d506a672d8eE9E5b41f9472`
* Blockchain Service URL: https://ropsten.infura.io/v3/fba92673ddcf4623bea97897935570bb

### Node 3 - Serving Node

* Earning Wallet: `0xB1E9823Cd2Fb8Bb33d506a672d8eE9E5b41f9472`
* Blockchain Service URL: https://ropsten.infura.io/v3/cbc31e02d9c346189a9e27dcea126dae

### Node 4 - Consuming Node - your VM where you will use YouTube or whatever through the network

* Consuming Wallet (Private Key): `a512fedae0722ff379558cb206af53bb5594902cc01a3ce8148786edb182dbdb`

## Starting the Nodes

In order to start the nodes you need a few details which will ultimately be baked into ARC or entered through a UI.  The cryptocurrency details above will be stripped out as mentioned so there are a few basic things:

1.  Earning Wallet (for serving nodes) - see above
2.  Blockchain Service URL - see above
3.  IP Address of the current machine - whatever the front facing IP address of the machine is, if you don't know your IP you can use the following command from Terminal to get it
    ```
    dig +short myip.opendns.com @resolver1.opendns.com
    ```
4.  DNS Servers - DNS server that you have access to.  As more nodes come on the network more options for routing become available for exit nodes to use to gain access to locations that are blocked to other nodes.  For the purposes of this demo we are using `9.9.9.9` for the DNS servers but you can use whatever you like...
5.  Consuming Wallet Private Key (for your final consuming node) - see above - again this will be stripped but this allows access to you consuming wallet by the network to pay your routing bill basically

### Node 1 - The Bootstrap Node - Ubuntu Server 1

Assuming you have followed the commands above and have a Ubuntu Server running and have compiled SubstratumNode from source the process is very short.

1.  Change directories to your substratum directory
    `cd $HOME/substratum`
2.  Ensure you know the variables you need, for the purposes of this demo we have embedded the Earning Wallet and Blockchain Service URL's provided in the section above in this command but you may change them if you like.  You will need (your IP address and your DNS servers if you wish to replace `9.9.9.9`)
3.  Run SubstratumNode

```
export RUST_BACKTRACE=1; \
sudo systemctl stop systemd-resolved.service && \
sudo ./SubstratumNode --earning-wallet 0xB1E9823Cd2Fb8Bb33d506a672d8eE9E5b41f9472 --dns-servers 9.9.9.9 --ip {YOUR_IP_ADDRESS} --neighborhood-mode standard --log-level debug &
```

4.  SubstratumNode will now run and provide you with your Substratum Node local descriptor, you will need to copy this to provide it to the next node joining the neighborhood - it will look something like:

`5qcrHWw50fH73iUgOlMAECDkmQRNn0alv2VmVpdWBio@172.104.198.10:9110`

### Node 2 - Standard Serving Node - Ubuntu Server 2

The command for standard serving nodes is slightly different as they are not STARTING a neighborhood but are JOINING a neighborhood.

1.  Change directories to your substratum directory
    `cd $HOME/substratum`
2.  Ensure you know the variables you need, for the purposes of this demo we have embedded the Earning Wallet and Blockchain Service URL's provided in the section above in this command but you may change them if you like.  You will need (your IP address and your DNS servers if you wish to replace `9.9.9.9`) and the node descriptor provided to you by SubstratumNode upon bootup in your Bootstrap Node above.
3.  Run SubstratumNode

```
export RUST_BACKTRACE=1; sudo systemctl stop systemd-resolved.service; sudo ./SubstratumNode --earning-wallet 0xB1E9823Cd2Fb8Bb33d506a672d8eE9E5b41f9472 --dns-servers 9.9.9.9 --ip {YOUR_IP_ADDRESS} --neighbors {BOOTSTRAP_NODE_DESCRIPTOR} --neighborhood-mode standard --log-level debug &
```

4.  SubstratumNode will now run and join the neighborhood you started in the above step on Server 1 and provide you with your Substratum Node local descriptor, you will need to copy this to provide it to the next node joining the neighborhood - it will look something like:

`5qcrHWw50fH73iUgOlMAECDkmQRNn0alv2VmVpdWBio@172.104.198.10:9110`

### Node 3 - Standard Serving Node - Ubuntu Server 3

The command for standard serving nodes is slightly different as they are not STARTING a neighborhood but are JOINING a neighborhood.

1.  Change directories to your substratum directory
    `cd $HOME/substratum`
2.  Ensure you know the variables you need, for the purposes of this demo we have embedded the Earning Wallet and Blockchain Service URL's provided in the section above in this command but you may change them if you like.  You will need (your IP address and your DNS servers if you wish to replace `9.9.9.9`) and the node descriptor provided to you by SubstratumNode upon bootup in your Bootstrap Node above, you do not need to provide it with the node descriptor from the bootstrap node ONLY the descriptor from Server 2.  Obviously as more and more nodes join the neighborhood the more obfuscated things get as each node only knows about one other node.
3.  Run SubstratumNode

```
export RUST_BACKTRACE=1; \
sudo systemctl stop systemd-resolved.service && \
sudo ./SubstratumNode --earning-wallet 0xB1E9823Cd2Fb8Bb33d506a672d8eE9E5b41f9472 --dns-servers 9.9.9.9 --ip {YOUR_IP_ADDRESS} --blockchain-service-url {BLOCKCHAIN_SERVICE_URL} --neighbors {NODE_DESCRIPTOR_FROM_SERVER_2} --neighborhood-mode standard --log-level debug &
```

4.  SubstratumNode will now run and join the neighborhood you started in the above step on Server 2 and provide you with your Substratum Node local descriptor, you will need to copy this to provide it to the next node joining the neighborhood - it will look something like:

`5qcrHWw50fH73iUgOlMAECDkmQRNn0alv2VmVpdWBio@172.104.198.10:9110`

### Node 4 - Consuming Node - Ubuntu Desktop

Now for the fun part, with the node descriptor from Server 3 you will be able to connect to the network neighborhood you just created.  While the minimal requirements are only 2 serving nodes + one boostrap node and one node to consume with as more and more nodes become servers and consumers the network becomes more anonymous, secure and obfuscated.

#### Turn on Substratum to Consume

1.  Change directories to your substratum directory
    `cd $HOME/substratum/releases`
2.  Ensure you know the variables you need, for the purposes of this demo we have embedded the Earning Wallet and Blockchain Service URL's provided in the section above in this command but you may change them if you like.  You will need (your IP address and your DNS servers if you wish to replace `9.9.9.9`) and the node descriptor provided to you by SubstratumNode upon bootup in your Bootstrap Node above, you do not need to provide it with the node descriptor from the bootstrap node ONLY the descriptor from Server 2.  Obviously as more and more nodes join the neighborhood the more obfuscated things get as each node only knows about one other node.
3.  Run SubstratumNode in CONSUME ONLY MODE - this node will not serve traffic but only consume it from other nodes

```
export RUST_BACKTRACE=1;  \
sudo systemctl stop systemd-resolved.service && \
sudo ./SubstratumNode --consuming-private-key a512fedae0722ff379558cb206af53bb5594902cc01a3ce8148786edb182dbdb --dns-servers 9.9.9.9 --ip {YOUR_IP_ADDRESS} --blockchain-service-url https://ropsten.infura.io/v3/30913bd15f5f41bcb1620ec123df5380  --neighbors {NODE_DESCRIPTOR_FROM_SERVER_3} --neighborhood-mode consume-only --log-level debug &
```

4.  SubstratumNode will now run and join the neighborhood you started in the above step on Server 2

#### Open a Second Terminal Window to Subvert Your DNS

1.  Ensure you are in the substratum/releases directory you created...
    `cd $HOME/substratum/releases`
2.  Take note of your IP address using the dig command below or navigating a browser to something like whatsmyip.org
    ```
    dig +short myip.opendns.com @resolver1.opendns.com
    ```
3.  Subvert your DNS to 127.0.0.1 - you can do this with dns_utility from the directory you are in with the following command
    `sudo sh ./dns_utility subvert`

    Once you subvert your DNS you will not receive any internet traffic unless consuming through the Substratum Network.  If you subvert your DNS BEFORE running the Substratum command above you will get no traffic at all...

## Open Your Browser

1.  Navigate to something like YouTube and try any video that is not live
2.  Navigate to whatsmyip.org - sometimes it takes a couple cycles but your IP will change
3.  You should be terminaled into all the other 3 servers and see traffic passing over them
